/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;

import org.ow2.petals.admin.api.artifact.lifecycle.ArtifactLifecycleFactory;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class PetalsAdministrationFactory {

    /**
     * Name of the property used to select the implementation of Petals
     * Administration API.
     */
    public final static String PROPERTY_NAME = PetalsAdministrationFactory.class.getName();

    /**
     * <p>
     * Create a new instance of Petals Administration API using a implementation
     * according to the following rule:
     * <ul>
     * <li>first, if a property with the service name is set, it is used to look
     * for the service implementation,</li>
     * <li>otherwise the service implementation is looking for in the current
     * classloader.</li>
     * </ul>
     * </p>
     * 
     * @throws DuplicatedServiceException
     *             The service to look for is duplicated.
     * @throws MissingServiceException
     *             Unable to find the service.
     */
    public static final PetalsAdministrationFactory newInstance()
            throws DuplicatedServiceException, MissingServiceException {
        final String overridenImplementation = System
                .getProperty(PetalsAdministrationFactory.PROPERTY_NAME);
        if (overridenImplementation == null || overridenImplementation.trim().isEmpty()) {
            final ServiceLoader<PetalsAdministrationFactory> serviceLoader = ServiceLoader
                    .load(PetalsAdministrationFactory.class);
            final Iterator<PetalsAdministrationFactory> servicesIterator = serviceLoader.iterator();
            try {
                if (servicesIterator.hasNext()) {
                    final PetalsAdministrationFactory serviceLoaded = servicesIterator.next();
                    if (servicesIterator.hasNext()) {
                        throw new DuplicatedServiceException(PetalsAdministrationFactory.class);
                    } else {
                        return serviceLoaded;
                    }
                } else {
                    throw new MissingServiceException(PetalsAdministrationFactory.class);
                }
            } catch (final ServiceConfigurationError e) {
                throw new MissingServiceException(PetalsAdministrationFactory.class, e);
            }
        } else {
            try {
                return ReflectionHelper.newInstance(overridenImplementation, null);
            } catch (final InvocationTargetException e) {
                throw new MissingServiceException(PetalsAdministrationFactory.class, e);
            }
        }
    }

    /**
     * Create instance of {@link ArtifactAdministration} implementation.
     * 
     * @return an {@link ArtifactAdministration} instance.
     */
    public abstract ArtifactAdministration newArtifactAdministration();

    /**
     * Create instance of {@link ContainerAdministration} implementation.
     * 
     * @return a {@link ContainerAdministration} instance.
     */
    public abstract ContainerAdministration newContainerAdministration();

    /**
     * Create instance of {@link RegistryAdministration} implementation.
     * 
     * @return a {@link RegistryAdministration} instance.
     */
    public abstract RegistryAdministration newRegistryAdministration();

    /**
     * Create instance of {@link ArtifactLifecycleFactory} implementation.
     * 
     * @return an {@link ArtifactLifecycleFactory} instance.
     */
    public abstract ArtifactLifecycleFactory newArtifactLifecycleFactory();

}
